# NOTE: symbols
THERMODYNAMIC_SYMBOLS = {
    "temperature": "T",
    "pressure": "P",
    "molar_volume": "MoVo",
    "critical_temperature": "Tc",
    "critical_pressure": "Pc",
    "critical_molar_volume": "Vc",
    "critical_compressibility_factor": "Zc",
    "molecular_weight": "MW",
    "boiling_temperature": "Tb",
    "melting_temperature": "Tm",
    "acentric_factor": "AcFa",
    "liquid_density": "rho_LIQ",
    "gas_density": "rho_G",
    "vapor_pressure": "VaPr",
    "enthalpy_of_formation": "EnFo",
    "Gibbs_energy_of_formation": "GiEnFo",
    "liquid_enthalpy_of_formation": "EnFo_LIQ",
    "liquid_Gibbs_energy_of_formation": "GiEnFo_LIQ",
    "liquid_entropy": "Ent_LIQ",
    "ideal_gas_enthalpy_of_formation": "EnFo_IG",
    "ideal_gas_Gibbs_energy_of_formation": "GiEnFo_IG",
    "ideal_gas_entropy": "Ent_IG",
    "ideal_gas_heat_capacity_at_constant_pressure": "Cp_IG",
    "liquid_heat_capacity_at_constant_pressure": "Cp_LIQ",
    "solid_heat_capacity": "Cp_SOL",
    "enthalpy_of_vaporization": "EnVap",
    "enthalpy_of_fusion": "EnFus",
    "standard_net_enthalpies_of_combustion": "EnCo_STD"
}
